/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;


import java.lang.ref.WeakReference;

import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.Spinner;
import android.widget.Toast;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.base.AbstractActivity;
import com.danielme.muspyforandroid.services.MuspyClient;
import com.danielme.muspyforandroid.utils.Utils;

/**
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class ImportLastfmActivity extends AbstractActivity
{

	/* ************************************************************ */
	/*							FIELDS            					*/
	/* ************************************************************ */

	private SeekBar seekBarTop;

	private Spinner spinnerPeriod;

	private EditText usernameEditText;
	
	private TextView textViewSeekBar;
	
	private AddHandler addHandler = new AddHandler(this);

	/* ************************************************************ */
	/*							METHODS            					*/
	/* ************************************************************ */

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		setContentView(R.layout.importlastfm);

		markButtonPressed(R.id.artists, true);

		seekBarTop = (SeekBar) findViewById(R.id.top);
		spinnerPeriod = (Spinner) findViewById(R.id.period);
		usernameEditText = (EditText) findViewById(R.id.username);
		textViewSeekBar = (TextView) findViewById(R.id.textViewSeekBar);
		textViewSeekBar.setText("25");
		String[] period = {getString(R.string.periodOverall), getString(R.string.period12month), getString(R.string.period6month), getString(R.string.period3month), getString(R.string.period7day) };
		spinnerPeriod.setAdapter(new ArrayAdapter<String>(this, R.layout.spinner, period));
		
		seekBarTop.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {            
          
           @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser)
           {
                  //min 25
                 textViewSeekBar.setText(progress + 25 + "");
                 
            }

			@Override
			public void onStartTrackingTouch(SeekBar arg0)
			{
				// nothing here
				
			}

			@Override
			public void onStopTrackingTouch(SeekBar arg0)
			{
				// nothing here
				
			}
     });

	}

	public void importArtists(View view)
	{

		if (!Utils.hasText(usernameEditText))
		{
			usernameEditText.setError(getString(R.string.nolastfm));
			//bug in 4.2
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				usernameEditText.requestFocus();
			}
		}
		else
		{
			usernameEditText.setError(null);

			if (checkConnection())
			{
				showProgressDialog(null);
				new Thread()
				{
					public void run()
					{ 
						try
						{
							String period = null;
							int pos = spinnerPeriod.getSelectedItemPosition();
							switch (pos)
							{
							case 0:
								period = Constants.LASTFM.OVERALL;
								break;
							case 1:
								period = Constants.LASTFM.MONTH12;
								break;
							case 2:
								period = Constants.LASTFM.MONTH6;
								break;
							case 3:
								period = Constants.LASTFM.MONTH3;
								break;
							case 4:
								period = Constants.LASTFM.DAY7;
								break;
							}
							MuspyClient.getInstance().importLastFm(getApplicationContext(), usernameEditText.getText().toString(), period, textViewSeekBar.getText().toString());
							addHandler.sendEmptyMessage(0);
						}
						catch (Exception ex)
						{
							Log.e(ReleasesActivity.class.toString(), ex.getMessage(), ex);
							Message message = new Message();
							message.obj = ex;
							message.what = 1;
							addHandler.sendMessage(message);
						}
					}
				}.start();
			}

		}

	}

	/**
	 * Shows artists screen and a Toast.
	 */	
	private static class AddHandler extends Handler
	{
		private final WeakReference<ImportLastfmActivity> myActivity;
        
        public AddHandler(ImportLastfmActivity myActivity)
         {
        	      this.myActivity = new WeakReference<ImportLastfmActivity>(myActivity);
         }   
        
        private ImportLastfmActivity getActivity()
        {
        	return myActivity.get();
        }
        
		@Override
		public void handleMessage(Message msg)
		{
			getActivity().getProgressDialog().dismiss();
			if (msg.what == 1)
			{
				getActivity().showError((Exception) msg.obj);
			}
			else
			{
				getActivity().showToast(R.string.importing, Toast.LENGTH_LONG);
				Intent intent = new Intent(getActivity(), MyArtistsActivity.class);
				getActivity().defaultTransition(intent);
			}
		}
	};
}