/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;

import java.net.URLEncoder;
import java.util.Calendar;
import java.util.HashMap;

import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.app.SearchManager;
import android.content.ActivityNotFoundException;
import android.content.ComponentName;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.provider.MediaStore;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.MuspyApplication;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.base.AbstractActivity;
import com.danielme.muspyforandroid.cache.CoverMemoryCache;
import com.danielme.muspyforandroid.model.Release;
import com.danielme.muspyforandroid.services.MusicBrainzClient;
import com.danielme.muspyforandroid.services.MuspyClient;
import com.danielme.muspyforandroid.utils.Utils;

/**
 * Details of a release.
 * 
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 * 
 */
public class ReleaseActivity extends AbstractActivity
{

	/* ************************************************************ */
	/* FIELDS */
	/* ************************************************************ */

	private ImageView coverImageView;

	private  Release releaseSelected;

	private static final HashMap<Character, Character> CHARS = new HashMap<Character, Character>() {
		private static final long serialVersionUID = -520870461677886510L;
		{
			put('á', 'a');
			put('é', 'e');
			put('í', 'i');
			put('ó', 'o');
			put('ú', 'u');
			put('à', 'a');
			put('è', 'e');
			put('ì', 'i');
			put('ò', 'o');
			put('ù', 'u');
			put('—', ' ');
			put('¡', ' ');
		}
	};

	private AlertDialog alertDialog;

	private LoadDetails loadDetails;

	private TextView textViewCountryContent;

	private TextView textViewLabelContent;

	private TextView textViewFormatContent;

	private ProgressBar progressBar;

	private TextView textViewTrackcountContent;

	/* ************************************************************ */
	/* OVERRIDE */
	/* ************************************************************ */

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		MuspyApplication muspyApplication = (MuspyApplication) getApplication();
		if (muspyApplication.getUserID() == null)
		{
			defaultTransition(new Intent(this, SignInActivity.class));
		}
		super.onCreate(savedInstanceState);

		if (getLastNonConfigurationInstance() != null)
		{
			releaseSelected = (Release) getLastNonConfigurationInstance();
		}
		else
		{	
			releaseSelected = (Release) getIntent().getParcelableExtra(Constants.RELEASE_SEL);
		}	

		
		setContentView(R.layout.release);
		setTitle(getString(R.string.app_name) + " - " + releaseSelected.getName());
		findViewById(R.id.imageButtonInfo).setTag(releaseSelected.getArtist().getMbid() + Constants.INFO_SEPARATOR + releaseSelected.getArtist().getName());

		markButtonPressed(R.id.releases, true);
		
		textViewCountryContent = (TextView) findViewById(R.id.textViewCountryContent);
		textViewCountryContent.setSelected(true);
		textViewLabelContent = (TextView) findViewById(R.id.textViewLabelContent);
		textViewLabelContent.setSelected(true);
		textViewFormatContent = (TextView) findViewById(R.id.textViewFormatContent);
		textViewFormatContent.setSelected(true);
		textViewTrackcountContent = (TextView) findViewById(R.id.textViewTrackcountContent);
		textViewTrackcountContent.setSelected(true);
		
		progressBar = (ProgressBar) findViewById(R.id.progressBarDetails);
		
		if (releaseSelected.getCountry() == null && releaseSelected.getFormat() == null && releaseSelected.getLabel() == null)
		{
			 progressBar.setVisibility(View.VISIBLE);
			 loadDetails = new LoadDetails();
			 loadDetails.execute("");			 			 
		}
		else
		{
			setDetails();
		}
		
		coverImageView = (ImageView) findViewById(R.id.cover);
		TextView artistTextView = (TextView) findViewById(R.id.artist);
		TextView dateTextView = (TextView) findViewById(R.id.date);
		TextView titleTextView = (TextView) findViewById(R.id.title);
				
		artistTextView.setText(releaseSelected.getArtist().getName());
		dateTextView.setText(releaseSelected.getDateLocale());
		titleTextView.setText("[" + releaseSelected.getType() + "] "+ releaseSelected.getName());
		titleTextView.setSelected(true);
		
		Drawable drawableCover = CoverMemoryCache.get(releaseSelected.getMbid());		

		if (drawableCover == null && checkConnection())
		{
			
			(new LoadImage()).execute();
		} 
		else
		{
			coverImageView.setImageDrawable(drawableCover);
		}

	}


	@Override
	public Object onRetainNonConfigurationInstance()
	{
		if (alertDialog != null && alertDialog.isShowing())
		{
			alertDialog.dismiss();
		}		
		if (getNoConnectionDialog() != null	&& getNoConnectionDialog().isShowing())
		{
			getNoConnectionDialog().dismiss();
		}
		if (loadDetails != null)
		{
			loadDetails.cancel(true);
		}
		
		return releaseSelected;
	}
	
	/**
	 * 
	 * @since 1.2
	 *
	 */
	private class LoadDetails extends AsyncTask<String, Void, String>
	{

		@Override
		protected String doInBackground(String... params)
		{
			MusicBrainzClient.getInstance().loadReleaseConcrete(releaseSelected);
			return null;
		}
		
		@Override
		protected void onPostExecute(String result)
		{
			 progressBar.setVisibility(View.GONE);			 

			setDetails();

		}
		
	}
	

	/* ************************************************************ */
	/* BUTTONS */
	/* ************************************************************ */

	public void mb(View view)
	{
		webBrowser(getMbLink());
	}

	/**
	 * Opens the default Youtube app.
	 */
	public void youtube(View view)
	{
		Intent intent = new Intent(Intent.ACTION_SEARCH);
		intent.setPackage("com.google.android.youtube");
		intent.putExtra("query", getQuery());
		intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		try
		{
			startActivity(intent);
		} 
		catch (ActivityNotFoundException e)
		{
			Builder builder = createInfoDialog(getString(R.string.noappyoutube));

			builder.setNeutralButton(R.string.ok, null);
			alertDialog = builder.setTitle(getString(R.string.infotitle)).create();
			alertDialog.show();
			dialogStyle(alertDialog);
		}

	}

	public void wikipedia(View view)
	{
		webBrowser("http://en.m.wikipedia.org/w/index.php?search=" + URLEncoder.encode(getQuery()));
	}

	public void lastfm(View view)
	{
		webBrowser("http://www.lastfm.com/search?q="+ URLEncoder.encode(getQuery()));
	}

	//TODO select country
	public void amazon(View view)
	{
		webBrowser("http://www.amazon.com/gp/aw/s/?k=" + URLEncoder.encode(getQuery()));
	}

	/**
	 * android.content.Intent.ACTION_SEND
	 */
	public void share(View view)
	{
		String message = releaseSelected.getArtist().getName() + " ( "+ releaseSelected.getDateLocale() + ") :  "	+ releaseSelected.getName() + "   -   " + getMbLink();
		Intent intent = new Intent(android.content.Intent.ACTION_SEND);

		intent.setType("text/plain");
		intent.putExtra(android.content.Intent.EXTRA_SUBJECT, R.string.share_title);
		intent.putExtra(Intent.EXTRA_TEXT, message);
		startActivityForResult(Intent.createChooser(intent, getString(R.string.share_title)),0);
	}
	
	/**
	 * http://stackoverflow.com/questions/8469317/send-song-title-to-spotify-to-start-playing-from-android-app
	 * @since 1.2
	 */
	public void spotify(View view)
	{
		Intent intent = new Intent(Intent.ACTION_MAIN);
		intent.setAction(MediaStore.INTENT_ACTION_MEDIA_PLAY_FROM_SEARCH);
		intent.setComponent(new ComponentName("com.spotify.mobile.android.ui", "com.spotify.mobile.android.ui.Launcher"));
		intent.putExtra(SearchManager.QUERY, getQuery());
		try
		{
			startActivity(intent);
		} 
		catch (ActivityNotFoundException e)
		{
			Builder builder = createInfoDialog(getString(R.string.noappspotify));
			builder.setNeutralButton(R.string.ok, null);
			alertDialog = builder.setTitle(getString(R.string.infotitle)).create();
			alertDialog.show();
			dialogStyle(alertDialog);
		}
	}

	/**
	 * @since 1.1
	 */
	public void calendar(View view)
	{ 
		Calendar calendar = releaseSelected.getCalendar();
		//year is not enough
		if (calendar == null)
		{
			Builder builder = createInfoDialog(getString(R.string.noreleaseday));			
			builder.setNeutralButton(getString(R.string.ok), null);	
			alertDialog = builder.create();
			alertDialog.show();
			dialogStyle(alertDialog);
		}
		else
		{
			Intent intent = new Intent(Intent.ACTION_EDIT);
			intent.setType("vnd.android.cursor.item/event");
			intent.putExtra("beginTime", calendar.getTimeInMillis());
			intent.putExtra("allDay", true);
			intent.putExtra("endTime", calendar.getTimeInMillis() + 36000);
			intent.putExtra("title", releaseSelected.getArtist().getName() + " : " + releaseSelected.getName());
			intent.putExtra("description", getMbLink());
			
			startActivity(Intent.createChooser(intent, getString(R.string.pickcalendar)));		
		}

	}


	/* ************************************************************ */
	/* PRIVATE */
	/* ************************************************************ */

	private String getMbLink()
	{
		return "http://musicbrainz.org/release-group/" + releaseSelected.getMbid();
	}

	private String getQuery()
	{
		String text = (releaseSelected.getArtist().getName() + " " + releaseSelected.getName()).toLowerCase();

		// Normalized is only available since Gingerbread :(
		/*
		 * text = text.replace("—", " "); String normalized =
		 * Normalizer.normalize(text, Normalizer.Form.NFD); return
		 * normalized.replaceAll("\\p{InCombiningDiacriticalMarks}+", "");
		 */
		char[] charArray = text.toCharArray();
		for (int i = 0; i < text.length(); i++)
		{
			Character character = CHARS.get(charArray[i]);
			if (character != null)
			{
				charArray[i] = character;
			}
		}
		return String.valueOf(charArray);
	}

	private void webBrowser(String url)
	{
		Intent intent = null;
		boolean useInternalBrowser = PreferenceManager.getDefaultSharedPreferences(this).getBoolean(Constants.PREF_BROWSER, true);

		if (useInternalBrowser)
		{
			intent = new Intent(this, WebBrowserActivity.class);
			Bundle bundle = new Bundle();
			bundle.putString(Constants.PARAM_URL, url);
			intent.putExtras(bundle);
		}
		else
		{
			 intent = new Intent(Intent.ACTION_VIEW, Uri.parse(url));
		}
		
		defaultTransition(intent);
	}

	/**
	 * Loads the cover and stores it in the cache.
	 * 
	 */
	private class LoadImage extends AsyncTask<String, Void, String>
	{
		private Drawable drawable;

		@Override
		protected String doInBackground(String... arg0)
		{
			try
			{			
				drawable = MuspyClient.getInstance().getCover(releaseSelected.getMbid());
				CoverMemoryCache.add(releaseSelected.getMbid(), drawable);				
			} 
			catch (Exception ex)
			{
				Log.e(ReleaseActivity.class.toString(), ex.getMessage(), ex);
			}
			return null;
		}

		@Override
		protected void onPostExecute(String result)
		{
			if (drawable != null)
			{
				coverImageView.setImageDrawable(drawable);
			} else
			{
				coverImageView.setImageResource(R.drawable.covernotavailable);
			}
		}

	}
	
	private void setDetails()
	{
		if (!Utils.hasText(releaseSelected.getCountry()))
		{
			textViewCountryContent.setText(getText(R.string.notavailableDetails));
		}
		else
		{
			textViewCountryContent.setText(releaseSelected.getCountry());
		}
		
		if (!Utils.hasText(releaseSelected.getLabel()))
		{
			textViewLabelContent.setText(getText(R.string.notavailableDetails));
		}
		else
		{
			textViewLabelContent.setText(releaseSelected.getLabel());
		}		
		
		if (!Utils.hasText(releaseSelected.getFormat()))
		{
			textViewFormatContent.setText(getText(R.string.notavailableDetails));
		}
		else
		{
			textViewFormatContent.setText(releaseSelected.getFormat());
		}
		
		if (!Utils.hasText(releaseSelected.getTrackCount()))
		{
			textViewTrackcountContent.setText(getText(R.string.notavailableDetails));
		}
		else
		{
			textViewTrackcountContent.setText(releaseSelected.getTrackCount());
		}
	}
	
}