/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;


import java.lang.ref.WeakReference;

import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.EditText;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.MuspyApplication;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.base.AbstractActivity;
import com.danielme.muspyforandroid.model.UserSettings;
import com.danielme.muspyforandroid.services.MuspyClient;
import com.danielme.muspyforandroid.utils.Utils;

/**
 * Main screen. Once logged, the only way to come back is by pressing "Sign out".
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class SignInActivity extends AbstractActivity
{
	/* ************************************************************ */
	/*							FIELDS            					*/
	/* ************************************************************ */

	private EditText emailEditText;
	private EditText passwordEditText;
	private UserSettings userSettings;
	
	private SignHandler signHandler = new SignHandler(this);

	/* ************************************************************ */
	/*							OVERRIDE           					*/
	/* ************************************************************ */

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		setTitle(getString(R.string.app_name_loading));
		
		super.onCreate(savedInstanceState);		
		MuspyApplication muspyApplication = (MuspyApplication) getApplicationContext();
		
		if (muspyApplication.getUserID() != null && muspyApplication.getEmail() != null && muspyApplication.getPass() != null)
		{			
			Intent intent = new Intent(this, ReleasesActivity.class);
			intent.putExtra(Constants.RELEASE_ENTRYPOINT, Boolean.TRUE);
			defaultTransition(intent);
		}
		else
		{
			setContentView(R.layout.signin);
			setTitle(getString(R.string.app_name_signin));
			emailEditText = (EditText) findViewById(R.id.editTextEmail);
			passwordEditText = (EditText) findViewById(R.id.password);			
		}
	}
	
	/**
	 * "Exits" (app remains in memory).
	 */
	@Override
	public void onBackPressed() 
	{
		//clean and beautiful hack ;)
        moveTaskToBack(true);   
    }

	/* ************************************************************ */
	/*							BUTTONS           					*/
	/* ************************************************************ */

	public void resetPass(View view)
	{
		Intent intent = new Intent(this, ResetPasswordActivity.class);
		Bundle bundle = new Bundle();
		bundle.putString(Constants.PARAM_EMAIL, emailEditText.getText().toString());
		intent.putExtras(bundle);
		defaultTransition(intent);
	}

	public void signin(View view) throws Exception
	{
		passwordEditText.setError(null);
		emailEditText.setError(null);
		emailEditText.setText(emailEditText.getText().toString().trim());

		if (!Utils.hasText(emailEditText))
		{
			emailEditText.setError(getString(R.string.noemail));
			//bug in 4.2
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				emailEditText.requestFocus();
			}
		}
		if (!Utils.hasText(passwordEditText))
		{
			passwordEditText.setError(getString(R.string.nopass));
			//bug in 4.2
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				passwordEditText.requestFocus();
			}
		}
		if (emailEditText.getError() == null && passwordEditText.getError() == null && checkConnection())
		{ 
			showProgressDialog(null);

			new Thread()
				{
					public void run()
					{
						try
						{
							userSettings =  MuspyClient.getInstance().getUserSettings(emailEditText.getText().toString(), passwordEditText.getText().toString());
							signHandler.sendEmptyMessage(0);
						}
						catch (Exception ex)
						{
							Log.e(SignInActivity.class.toString(), ex.getMessage(), ex);
							signHandler.sendEmptyMessage(1);
						}
					}

				}.start();
		}
	}

	private static class SignHandler extends Handler
	{
		
		private final WeakReference<SignInActivity> myActivity;
        
        public SignHandler(SignInActivity myActivity)
         {
        	      this.myActivity = new WeakReference<SignInActivity>(myActivity);
         }   
        
        private SignInActivity getActivity()
        {
        	return myActivity.get();
        }
        
		@Override
		public void handleMessage(Message msg)
		{
			getActivity().getProgressDialog().dismiss();

			if (getActivity().userSettings == null || msg.what == 1)
			{
				Builder builder = new Builder(getActivity());
				String message = null;
				if (msg.what == 1)
				{
					message = getActivity().getString(R.string.exception);
				}
				else
				{
					message = getActivity().getString(R.string.loginincorrect);
				}
				builder.setMessage(message);
				builder.setNeutralButton(R.string.ok, null);
				builder.setTitle(getActivity().getString(R.string.error));
				builder.setCancelable(false);
				builder.setIcon(android.R.drawable.ic_dialog_alert);
				AlertDialog alertDialog = builder.create();
				alertDialog.show();
				alertDialog.setCancelable(false);
				getActivity().dialogStyle(alertDialog);
			}
			else
			{				
				MuspyApplication muspyApplication =(MuspyApplication) getActivity().getApplication();
				muspyApplication.setCredentials(getActivity().emailEditText.getText().toString(), getActivity().passwordEditText.getText().toString(), getActivity().userSettings.getId());

				Intent intent = new Intent(getActivity(), ReleasesActivity.class);
				intent.putExtra(Constants.RELEASE_ENTRYPOINT, Boolean.TRUE);
				getActivity().defaultTransition(intent);
			}
		}
	};

	public void signup(View view)
	{
		passwordEditText.setError(null);
		emailEditText.setError(null);
		Intent intent = new Intent(SignInActivity.this, SignUpActivity.class);
		defaultTransition(intent);
	}


}