/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.model;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;

import android.os.Parcel;
import android.os.Parcelable;

import com.danielme.muspyforandroid.Constants;


/**
 * 
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class Release extends MusicBrainzID implements Parcelable
{
	private String year;
	private String month;
	private String day;
	private String type;
	private String name;
	private Artist artist;
	
	private String mbidConcreteRelease;
	private String format;
	private String label;
	private String country;
	private String trackCount;
	
	
	public static final Parcelable.Creator<Release> CREATOR =
		             new Parcelable.Creator<Release>()
		 	            {
		 	                @Override
		 	                public Release createFromParcel(Parcel parcel)
		 	                {
		 	                    return new Release(parcel);
		 	                }
		 
		 	                @Override
		 	                public Release[] newArray(int size)
		 	                {
		 	                    return new Release[size];
		 	                }
		 	        };


	public Release(Parcel parcel)
	{
		super();
		setMbid(parcel.readString());
		year = parcel.readString();
		month = parcel.readString();
		day = parcel.readString();
		type = parcel.readString();
		name = parcel.readString();
		artist = parcel.readParcelable(Artist.class.getClassLoader());
		format = parcel.readString();
		label = parcel.readString();
		country = parcel.readString();
		mbidConcreteRelease = parcel.readString();
		trackCount = parcel.readString();
	}

	public Release()
	{
		super();
	}

	public String getDateLocale()
	{
		StringBuilder date = new StringBuilder();
		if (Locale.getDefault().getLanguage().equals(Constants.LOCALE_ES))
		{
			if (day != null)
			{
				date.append(day + "-");
			}
			if (month != null)
			{
				date.append(month + "-");
			}
			date.append(year);
		}
		else
		{
			date.append(year);
			if (month != null)
			{
				date.append("-" + month);
				if (day != null)
				{
					date.append("-" + day);
				}
			}
		}
		
		return date.toString();
	}


	public String getType()
	{
		return type;
	}

	public void setType(String type)
	{
		this.type = type;
	}

	public String getName()
	{
		return name;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public Artist getArtist()
	{
		return artist;
	}

	public void setArtist(Artist artist)
	{
		this.artist = artist;
	}
	

	public String getYear()
	{
		return year;
	}


	public void setYear(String year)
	{
		this.year = year;
	}


	public String getMonth()
	{
		return month;
	}


	public void setMonth(String month)
	{
		this.month = month;
	}


	public String getDay()
	{
		return day;
	}


	public void setDay(String day)
	{
		this.day = day;
	}
	
	

	public String getMbidConcreteRelease()
	{
		return mbidConcreteRelease;
	}

	public void setMbidConcreteRelease(String mbidConcreteRelease)
	{
		this.mbidConcreteRelease = mbidConcreteRelease;
	}

	public String getFormat()
	{
		return format;
	}

	public void setFormat(String format)
	{
		this.format = format;
	}

	public String getLabel()
	{
		return label;
	}

	public void setLabel(String label)
	{
		this.label = label;
	}

	public String getCountry()
	{
		return country;
	}

	public void setCountry(String country)
	{
		this.country = country;
	}	
	

	public String getTrackCount()
	{
		return trackCount;
	}

	public void setTrackCount(String trackCount)
	{
		this.trackCount = trackCount;
	}

	public Calendar getCalendar()
	{
		Calendar calendar = null;
		calendar = new GregorianCalendar();
		
		if (month != null)
		{
			calendar.set(Calendar.YEAR, Integer.parseInt(year));
			calendar.set(Calendar.MONTH, Integer.parseInt(month) - 1);
			if (day != null)
			{
				calendar.set(Calendar.DAY_OF_MONTH, Integer.parseInt(day));
			}
			else
			{
				calendar.set(Calendar.DAY_OF_MONTH, 1);
			}
		}
		return calendar;		
	}


	@Override
	public int describeContents()
	{
		return 0;
	}

	@Override
	public void writeToParcel(Parcel parcel, int flags)
	{
		parcel.writeString(getMbid());
		parcel.writeString(year);
		parcel.writeString(month);
		parcel.writeString(day);
		parcel.writeString(type);
		parcel.writeString(name);
		parcel.writeParcelable(artist, 0);
		parcel.writeString(format);
		parcel.writeString(label);
		parcel.writeString(country);
		parcel.writeString(mbidConcreteRelease);
		parcel.writeString(trackCount);
	}


}